package com.emsoft.common

import com.emsoft.collections.CollectionHelper
import java.util.regex.Matcher
import java.util.regex.Pattern

/**
 * Utils for strings.
 *
 * @author Vladimir Zhukov
 */
class StringHelper {

    /**
     * Empty string - "".
     */
    public final static String EMPTY = ""
    /**
     * CR - "\r\n".
     */
    //public final static String CR = "\r\n"
    public final static String CR = "\n"

    protected StringHelper() {
    }

    /**
     * If string is null or empty then returns false.
     *
     * @param value
     * @return
     */
    static boolean hasText(String value, boolean ignoreWhiteSpace = false) {
        value != null && !(ignoreWhiteSpace ? value.trim() : value).empty
    }

    /**
     * Connects elements as strings.
     *
     * @param < T >
     * @param list
     * @param separator
     * @return
     */
    static String join(Iterable<?> list, String separator = EMPTY) {
        if (list == null)
            return EMPTY
        def hasSeparator = hasText(separator)
        def sb = new StringBuilder()
        list.each {
            def str = it as String
            if (hasText(str)) {
                if (sb.length() > 0 && hasSeparator)
                    sb.append(separator)
                sb.append(str)
            }
        }
        return sb.toString()
    }

    /**
     * Connects elements as strings.
     *
     * @param < T >
     * @param list
     * @return
     */
    static <T> String join(T... list) {
        return join(Arrays.asList(list), EMPTY)
    }

    /**
     * Split source by pattern(separators), includes in result separators.
     * Empty string doesn't not return.
     *
     * @param s
     * @param pattern
     * @return
     */
    static List<String> split(String source, String pattern, boolean includeSeparator = true) {
        Assert.hasText(source, "source")
        Assert.hasText(pattern, "pattern")
        return split(source, Pattern.compile(pattern), includeSeparator)
    }

    /**
     * Split source by pattern(separators), includes in result separators.
     * Empty string doesn't not return.
     *
     * @param s
     * @param pattern
     * @return
     */
    static List<String> split(String source, Pattern pattern, boolean includeSeparator = true) {
        Assert.hasText(source, "source")
        Assert.notNull(pattern, "pattern")
        Matcher m = pattern.matcher(source)
        List<String> list = new ArrayList<String>()
        int start = 0
        while (m.find()) {
            // separator, which found by pattern
            String delim = m.group()
            // part of string, which separate separator
            String token = source.substring(start, m.start())
            if (hasText(token))
                list.add(token)
            if (includeSeparator)
                list.add(delim)
            start = m.end()
        }
        if (start < source.length())
            list.add(source.substring(start))
        return list
    }

    /**
     *  Tokenize a String based on the given string delimiter and skip given indexes.
     * @param source
     * @param delimiter
     * @param skipIndexes
     * @return
     */
    static List<String> tokenize(String source, String delimiter, int ... skipIndexes) {
        Assert.hasText(source, "source")
        Assert.hasText(delimiter, "delimiter")
        Assert.hasElements(skipIndexes, "skipIndexes")
        def list = source.tokenize(delimiter)
        return CollectionHelper.excludeIndexes(list, skipIndexes)
    }

    /**
     * 1 - если оканчивается на цифру 1, кроме случаев, если оканчивается на 11.
     * 2 - если оканчивается на цифры 2..4.
     * 3 - если оканчивается на цифры 5..20.
     * @param number
     * @return
     */
    static int getPluralFormRU(int number) {
        def d10 = number % 10
        def d100 = number % 100
        return (d10 == 1 && d100 != 11 ? 1 : d10 >= 2 && d10 <= 4 && (d100 < 10 || d100 >= 20) ? 2 : 3)
    }

    /**
     *
     * @param number
     * @param form1 если оканчивается на цифру 1, кроме случаев, если оканчивается на 11.
     * @param form2 если оканчивается на цифры 2..4.
     * @param form3 если оканчивается на цифры 5..20.
     * @return
     */
    static String getPluralFormRU(int number, String form1, String form2, String form3) {
        def form = getPluralFormRU(number)
        if (form == 1)
            return form1
        if (form == 2)
            return form2
        return form3
    }

    static int toInteger(String value) {
        if (value)
            return value.toInteger()
        return 0
    }

    static String toFirstLowerCase(String value) {
        if (!value)
            return value
        value[0].toLowerCase() + value.substring(1)
    }
}