﻿using System.Collections.Generic;
using System.Configuration;
using Alseco.Utils;

namespace Alseco.Configuration {

    public abstract class ConfigurationBase : ConfigurationHelper {

        public const string ApplicationSettingsGroupName = "applicationSettings";
        public const string UserSettingsGroupName = "userSettings";
        public string DefaultSectionName { get; protected set; }
        public string SectionGroupName { get; protected set; }
        public System.Configuration.Configuration Configuration { get; protected set; }
        public ConfigurationUserLevel Level { get; protected set; }

        protected ConfigurationBase(System.Configuration.Configuration configuration,
            ConfigurationUserLevel level, string sectionGroupName,
            string defaultSectionName) {
            ExceptionHelper.CheckOnNull(configuration, "configuration");
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionGroupName,
                "sectionGroupName");
            ExceptionHelper.CheckStringOnNullOrEmpty(defaultSectionName,
                "defaultSectionName");
            Configuration = configuration;
            Level = level;
            SectionGroupName = sectionGroupName;
            DefaultSectionName = defaultSectionName;
        }

        protected ConfigurationBase(string configFileFullName,
            ConfigurationUserLevel level, string sectionGroupName,
            string defaultSectionName)
            : this(OpenConfiguration(configFileFullName, level),
            level, sectionGroupName, defaultSectionName) {
        }

        protected ConfigurationBase(string configFileFullName,
            ConfigurationUserLevel level, string defaultSectionName)
            : this(configFileFullName, level, (level == ConfigurationUserLevel.None ?
            ApplicationSettingsGroupName : UserSettingsGroupName), defaultSectionName) {
        }

        protected ConfigurationBase(ConfigurationUserLevel level, string defaultSectionName)
            : this(string.Empty, level, defaultSectionName) {
        }

        protected virtual void ConfigureSection(ConfigurationSection section) {
            ConfigureSection(section, Level);
        }

        public void WriteConfiguration<TElement, TSection>(string sectionName,
            IEnumerable<TElement> elements,
            AddElementsHandler<TElement, TSection> addElementsHandler)
            where TElement : ConfigurationElement
            where TSection : ConfigurationSection, new() {
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionName, "sectionName");
            ExceptionHelper.CheckOnNull(elements, "elements");
            ExceptionHelper.CheckOnNull(addElementsHandler, "addElementsHandler");
            ConfigurationSectionGroup group =
                (Level == ConfigurationUserLevel.None
                ? (ConfigurationSectionGroup)OpenSectionGroup<ApplicationSettingsGroup>(Configuration, SectionGroupName)
                : (ConfigurationSectionGroup)OpenSectionGroup<UserSettingsGroup>(Configuration, SectionGroupName));
            TSection section = OpenSection<TSection>(group, Level, sectionName);
            addElementsHandler(section, elements);
        }

        public void WriteConfiguration<TElement, TSection>(IEnumerable<TElement> elements,
            AddElementsHandler<TElement, TSection> addElementHandler)
            where TElement : ConfigurationElement
            where TSection : ConfigurationSection, new() {
            WriteConfiguration<TElement, TSection>(DefaultSectionName, elements, addElementHandler);
        }

        public void SaveConfiguration<TElement, TSection>(string sectionName,
            IEnumerable<TElement> elements,
            AddElementsHandler<TElement, TSection> addElementsHandler)
            where TElement : ConfigurationElement
            where TSection : ConfigurationSection, new() {
            WriteConfiguration<TElement, TSection>(sectionName, elements,
                addElementsHandler);
            Configuration.Save();
            //ConfigurationManager.RefreshSection(sectionName);
        }

        public void SaveConfiguration<TElement, TSection>(IEnumerable<TElement> elements,
            AddElementsHandler<TElement, TSection> addElementHandler)
            where TElement : ConfigurationElement
            where TSection : ConfigurationSection, new() {
            SaveConfiguration<TElement, TSection>(DefaultSectionName, elements, addElementHandler);
        }

        public ICollection<TElement> LoadConfiguration<TElement, TSection>(
            string sectionName, GetElementsHandler<TElement, TSection> getElementsHandler)
            where TElement : ConfigurationElement
            where TSection : ConfigurationSection, new() {
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionName, "sectionName");
            ExceptionHelper.CheckOnNull(getElementsHandler, "getElementsHandler");
            List<TElement> elements = new List<TElement>();
            TSection section = OpenSection<ConfigurationSectionGroup, TSection>(
                Configuration, Level, SectionGroupName, sectionName, false);
            if (section == null)
                return elements;
            elements.AddRange(getElementsHandler(section));
            return elements;
        }
    }
}