﻿using System.Configuration;
using Alseco.Utils;

namespace Alseco.Configuration {

    public class ConfigurationHelper {

        protected ConfigurationHelper() {
        }

        public static System.Configuration.Configuration OpenConfiguration(
            string configFileFullName, ConfigurationUserLevel level) {
            if (string.IsNullOrEmpty(configFileFullName))
                return ConfigurationManager.OpenExeConfiguration(level);
            else {
                ExeConfigurationFileMap map = new ExeConfigurationFileMap();
                if (level == ConfigurationUserLevel.None)
                    map.ExeConfigFilename = configFileFullName;
                else if (level == ConfigurationUserLevel.PerUserRoaming)
                    map.RoamingUserConfigFilename = configFileFullName;
                else if (level == ConfigurationUserLevel.PerUserRoamingAndLocal)
                    map.LocalUserConfigFilename = configFileFullName;
                return ConfigurationManager.OpenMappedExeConfiguration(map, level);
            }
        }

        public static System.Configuration.Configuration OpenConfiguration(
            ConfigurationUserLevel level) {
            return OpenConfiguration(string.Empty, level);
        }

        public static T OpenSectionGroup<T>(
            System.Configuration.Configuration config,
            string sectionGroupName, bool createIfNotExist)
            where T : ConfigurationSectionGroup, new() {
            ExceptionHelper.CheckOnNull(config, "config");
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionGroupName,
                "sectionGroupName");
            T group = null;
            if (config.SectionGroups[sectionGroupName] == null) {
                if (createIfNotExist) {
                    group = new T();
                    config.SectionGroups.Add(sectionGroupName, group);
                }
            }
            else
                group = (T)config.SectionGroups[sectionGroupName];
            return group;
        }

        public static T OpenSectionGroup<T>(
            System.Configuration.Configuration config,
            string sectionGroupName)
            where T : ConfigurationSectionGroup, new() {
            return OpenSectionGroup<T>(config, sectionGroupName, true);
        }

        public static void ConfigureSection(ConfigurationSection section,
            ConfigurationUserLevel level) {
            ExceptionHelper.CheckOnNull(section, "section");
            section.SectionInformation.AllowExeDefinition =
                (level == ConfigurationUserLevel.None
                ? ConfigurationAllowExeDefinition.MachineToApplication
                : (level == ConfigurationUserLevel.PerUserRoaming
                    ? ConfigurationAllowExeDefinition.MachineToRoamingUser
                    : ConfigurationAllowExeDefinition.MachineToLocalUser));
            section.SectionInformation.RequirePermission = false;
        }


        public static T OpenSection<T>(ConfigurationSectionGroup sectionGroup,
            ConfigurationUserLevel level, string sectionName, bool createIfNotExist)
            where T : ConfigurationSection, new() {
            ExceptionHelper.CheckOnNull(sectionGroup, "sectionGroup");
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionName,
                "sectionName");
            T section = null;
            if (sectionGroup.Sections[sectionName] == null) {
                if (createIfNotExist) {
                    section = new T();
                    ConfigureSection(section, level);
                    sectionGroup.Sections.Add(sectionName, section);
                }
            }
            else
                section = (T)sectionGroup.Sections[sectionName];
            return section;
        }

        public static T OpenSection<T>(ConfigurationSectionGroup sectionGroup,
            ConfigurationUserLevel level, string sectionName)
            where T : ConfigurationSection, new() {
            return OpenSection<T>(sectionGroup, level, sectionName, true);
        }

        public static TSection OpenSection<TGroup, TSection>(
            System.Configuration.Configuration config, ConfigurationUserLevel level,
            string sectionGroupName, string sectionName, bool createIfNotExist)
            where TGroup : ConfigurationSectionGroup, new()
            where TSection : ConfigurationSection, new() {
            ExceptionHelper.CheckOnNull(config, "config");
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionGroupName, "sectionGroupName");
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionName, "sectionName");
            TGroup g = OpenSectionGroup<TGroup>(config,
                sectionGroupName, createIfNotExist);
            if (g != null)
                return OpenSection<TSection>(g, level, sectionName, createIfNotExist);
            return null;
        }

        public static TSection OpenSection<TSection>(
            System.Configuration.Configuration config, ConfigurationUserLevel level,
            string sectionGroupName, string sectionName, bool createIfNotExist)
            where TSection : ConfigurationSection, new() {
            ExceptionHelper.CheckOnNull(config, "config");
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionGroupName, "sectionGroupName");
            ExceptionHelper.CheckStringOnNullOrEmpty(sectionName, "sectionName");
            return (level == ConfigurationUserLevel.None
                ? OpenSection<ApplicationSettingsGroup, TSection>(
                config, level, sectionGroupName, sectionName, createIfNotExist)
                : OpenSection<UserSettingsGroup, TSection>(
                config, level, sectionGroupName, sectionName, createIfNotExist));
        }
    }
}