package com.emsoft.exceptions

import com.emsoft.common.Assert
import com.emsoft.common.ResourceHelper
import com.emsoft.common.StringHelper
import org.slf4j.LoggerFactory

/**
 * Utils for exceptions.
 *
 * @author Vladimir Zhukov
 */
class ExceptionHelper {

    /**
     * Resource file of error messages.
     */
    final static String ERROR_MESSAGES_RESOURCE = "com/emsoft/utils/resources/ErrorMessages"
    final static String DEFAULT_LOGGER = "default_error_logger"

    protected ExceptionHelper() {
    }

    /**
     * Returns text description of error from resources.
     *
     * @param key
     * @return
     */
    private static String getLocalExceptionMessage(String key) {
        ResourceBundle rb = ResourceHelper.getResourceBundleSafe(ERROR_MESSAGES_RESOURCE)
        if (rb == null)
            throwException(new InternalException(ErrorCode.INTERNAL_ERROR,
                    "Resource '${ERROR_MESSAGES_RESOURCE}.${ResourceHelper.BUNDLE_EXTENSION}' for error messages not found."))
        return rb.getString(key)
    }

    /**
     * Returns text description of error.
     *
     * @param ex
     * @param includeInnerException
     * @param includeStackTrace
     * @return
     */
    static String getExceptionMessage(Throwable ex, boolean includeInnerException = true,
            boolean includeStackTrace = false) {
        Assert.notNull(ex, "ex")
        StringBuilder sb = new StringBuilder()
        appendExceptionMessage(sb, ex)
        if (includeInnerException) {
            Throwable innerEx = ex.cause
            def tab = "  "
            while (innerEx != null) {
                appendExceptionMessage(sb, innerEx, tab)
                innerEx = innerEx.cause
                tab += "  "
            }
        }
        if (includeStackTrace)
            fillStackTrace(sb, ex, 0)
        return sb.toString()
    }

    private static void appendExceptionMessage(StringBuilder sb, Throwable ex, String tab = StringHelper.EMPTY) {
        if (sb.length() > 0) {
            sb.append(StringHelper.CR)
            sb.append(tab)
        }
        sb.append(ex.class.simpleName)
        sb.append(": ")
        sb.append(ex.localizedMessage)
    }

    /**
     * Returns stacktrace.
     *
     * @param ex
     * @return
     */
    static String getStackTrace(Throwable ex, int stackTraceDepth = 0) {
        Assert.notNull(ex, "ex")
        Assert.notNull(stackTraceDepth, "stackTraceDepth")
        def sb = new StringBuilder()
        fillStackTrace(sb, ex, stackTraceDepth)
        return sb.toString()
    }

    private static fillStackTrace(StringBuilder sb, Throwable ex, int stackTraceDepth) {
        StackTraceElement[] ar = ex.stackTrace
        if (ar) {
            if (sb.length() > 0)
                sb.append(StringHelper.CR)
            sb.append("StackTrace: ")
            int depth = (stackTraceDepth > 0 ? Math.min(stackTraceDepth, ar.length) : ar.length)
            def tab = StringHelper.EMPTY
            for (i in 0..depth - 1) {
                if (i > 0) {
                    sb.append(StringHelper.CR)
                    sb.append(tab)
                }
                sb.append(ar[i].toString())
                tab += "  "
            }
        }
    }

    /**
     * Returns first element of stack trace.
     *
     * @param ex
     * @return
     */
    static StackTraceElement getRootStackTrace(Throwable ex) {
        Assert.notNull(ex, "ex")
        StackTraceElement[] stAr = ex.stackTrace
        if (stAr)
            return stAr[stAr.length - 1]
        return null
    }

    /**
     * Returns class name of the first element of stack trace in which the exception occurred.
     *
     * @param ex
     * @return
     */
    static String getRootClassName(Throwable ex) {
        StackTraceElement trace = getRootStackTrace(ex)
        if (trace == null)
            return StringHelper.EMPTY
        return trace.getClassName()
    }

    /**
     * Throw ex.
     *
     * @param ex
     * @throws <T>
     */
    static <T extends Throwable> void throwException(T ex) {
        throw ex
    }

    static void throwException(String messageCode, Object... params) {
        throw new InternalException(ErrorCode.INTERNAL_ERROR, String.format(
                getLocalExceptionMessage(messageCode), params))
    }

    /**
     * Logged exception with slf4j, wrap in BaseRuntimeException if it not instance of RuntimeException and throw.
     *
     * @param ex
     * @param className
     */
    static void logAndThrow(Throwable ex, String className = getRootClassName(ex)) {
        Assert.hasText(className, "className")

        LoggerFactory.getLogger(className).error(getExceptionMessage(ex), ex)

        def runtimeEx = ex instanceof RuntimeException ? ex : new InternalException(ErrorCode.INTERNAL_ERROR, ex)
        throwException(runtimeEx)
    }

    static void throwNullArgument(String name) {
        throwException(new IllegalArgumentException(String.format(
                getLocalExceptionMessage("NullArgumentExceptionMessage"), name)))
    }

    static void throwStringArgumentNullOrEmpty(String name) {
        throwException(new IllegalArgumentException(String.format(
                getLocalExceptionMessage("StringArgumentNullOrEmptyMessage"), name)))
    }

    static void throwValueAlreadyExists(Object value) {
        throwException("ValueAlreadyExistsMessage", value)
    }

    static void throwObjectNotInCollection(String name) {
        throwException(new ObjectNotInCollectionException(String.format(
                getLocalExceptionMessage("ObjectNotInCollectionExceptionMessage"), name)))
    }

    static void throwIndexOutOfBounds() {
        throwException(new IndexOutOfBoundsException(
                getLocalExceptionMessage("ObjectNotInCollectionExceptionMessage")))
    }

    static void throwEmptyCollection(String col) {
        throwException(new EmptyCollectionException(col))
    }

    static void throwUnsupportedOperation() {
        throwException(new UnsupportedOperationException(
                getLocalExceptionMessage("UnsupportedOperationExceptionMessage")))
    }

    static void throwUnknownType(String type) {
        throwException(new InternalException(ErrorCode.INTERNAL_ERROR, String.format(
                getLocalExceptionMessage("UnknownTypeExceptionMessage"), type)))
    }

    static void throwResourceNotFound(String bundleName) {
        throwException(new ResourceNotFoundException(String.format(
                getLocalExceptionMessage("ResourceNotFoundExceptionMessage"), bundleName)))
    }

    static void throwPropertyNotInitialized(String name) {
        throwException(new InternalException(ErrorCode.INTERNAL_ERROR, String.format(
                getLocalExceptionMessage("PropertyNotInitializedExceptionMessage"), name)))
    }
}
