package com.emsoft.security

import com.emsoft.common.Assert
import com.emsoft.common.StringHelper
import com.emsoft.exceptions.ExceptionHelper
import java.security.MessageDigest
import java.security.NoSuchAlgorithmException
import java.security.SecureRandom
import org.codehaus.groovy.runtime.EncodingGroovyMethods

/**
 * Class for work with hash.
 *
 * @author Vladimir Zhukov
 */
class HashHelper {

    static final String SHA256 = "SHA-256"
    static final String SHA512 = "SHA-512"
    
    protected HashHelper() {
    }

    /**
     * Compares hash with hash of clearText.
     *
     * @param clearText
     * @param salt
     * @param hash
     * @return
     */
    boolean verify(String clearText, byte[] salt, byte[] hash, String algorithm = SHA256) {
        Assert.hasText(clearText, "clearText")
		Assert.hasText(algorithm, "algorithm")
        return verify(hash(clearText, salt, algorithm), hash)
    }

    /**
     * Compares hash with hash of clearText.
     *
     * @param clearText
     * @param salt
     * @param hash
     * @return
     */
    boolean verify(char[] clearText, byte[] salt, byte[] hash, String algorithm = SHA256) {
        Assert.hasElements(clearText, "clearText")
		Assert.hasText(algorithm, "algorithm")
        return verify(hash(clearText, salt, algorithm), hash)
    }

    boolean verify(byte[] hash1, byte[] hash2) {
        if (hash1 == null || hash2 == null || hash1.length != hash2.length)
            return false
        return Arrays.equals(hash1, hash2)
    }

    static byte[] hash(String clearText, String salt, String algorithm = SHA256) {
        Assert.hasText(clearText, "clearText")
        Assert.hasText(algorithm, "algorithm")
        def saltData = (salt ? EncodingGroovyMethods.decodeBase64(salt) : null)
        return hash(clearText, salt, algorithm)
    }

    /**
     * Returns hash of clearText.
     *
     * @param clearText
     * @param salt
     * @return
     */
    static byte[] hash(char[] clearText, byte[] salt, String algorithm = SHA256) {
        Assert.hasElements(clearText, "clearText")
        Assert.hasText(algorithm, "algorithm")
        return hash(String.valueOf(clearText), salt, algorithm)
    }

    /**
     * Returns hash of clearText.
     *
     * @param clearText
     * @param salt
     * @return
     */
    static byte[] hash(String clearText, byte[] salt, String algorithm = SHA256) {
        final String encoding = "UTF8"
        byte[] pwData = null
        try {
            pwData = clearText.getBytes(encoding)
        }
        catch (UnsupportedEncodingException ex) {
            ExceptionHelper.logAndThrow(ex, Password.class.getName())
        }
        int saltLength = (salt != null ? salt.length : 0)
        // create working array of sufficient size to accommodate all bytes
        byte[] data = new byte[saltLength + pwData.length]
        if (saltLength > 0) {
            // copy saltBytes in working array
            System.arraycopy(salt, 0, data, 0, salt.length)
        }
        // copy password in working array, converting it to UTF-8
        System.arraycopy(pwData, 0, data, saltLength, pwData.length)
        Byte[] hash = null
        // calculate hashBytes value
        try {
            MessageDigest md = MessageDigest.getInstance(algorithm)
            hash = md.digest(data)
        }
        catch (NoSuchAlgorithmException ex) {
            ExceptionHelper.logAndThrow(ex, Password.class.getName())
        }
        finally {
            // clear working array at end of work to avoid leakage of open password
            Arrays.fill(data, (Byte) 0)
        }
        return hash
    }

    /**
     * Generates array of random bytes.
     *
     * @param size
     * @return
     */
    static byte[] generateRandomBytes(int size) {
        byte[] random = new byte[size]
        new SecureRandom().nextBytes(random)
        return random
    }
}
